/* pgmpeg2.cc
 * ----------
    PGMPEG - MPEG1 player library for ParaGUI
    Copyright (C) 2002  Eric Ross, Alexander Pipelka, Teunis Peters

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Eric Ross <eross@disc.ucn.cl>
    Alexander Pipelka <pipelka@teleweb.at>
    Teunis Peters <teunis@greycloaklabs.ca>
 */

#include "pgmpeg2.hh"

/* entirely arbitrary.  Probably smarter to make it a variable inside the class
   or something like that */
#define PLAYERSCALE 1000

/* notes from Teunis Peters:
 * it might be an idea to move the 'frameupdate' function into the functions
 * Play(), ...
 * but I haven't yet seen the point - the current method makes a little more
 * sense from the perspective that the PG_MPEGPlayer doesn't know about the
 * widgets.
 */

void PG_MPEGPlayerWindow::handle_position(PG_Slider* slider) {
  pos = slider;
  if (pos == NULL) return;
  double v = currenttime() / totaltime();
  pos->SetRange(0,PLAYERSCALE);
  pos->SetPosition(v * PLAYERSCALE);
};

bool PG_MPEGPlayerWindow::handle_start(PG_Button* button) {
  // int id = button->GetID();
  Play();
  frameupdate();
  return true;
};

bool PG_MPEGPlayerWindow::handle_stop(PG_Button* button) {
  Stop();
  frameupdate();
  return true;
};

bool PG_MPEGPlayerWindow::handle_rewind(PG_Button* button) {
  SMPEGstatus v = cstat();
  Rewind();
  if (v == SMPEG_PLAYING) Play();
  frameupdate();
  return true;
};

bool PG_MPEGPlayerWindow::handle_pause(PG_Button* button) {
  Pause();
  frameupdate();
  return true;
};

bool PG_MPEGPlayerWindow::handle_transparency(PG_Slider* slider, long data) {
  SetTransparency(data);
  Update();
  return true;
}

bool PG_MPEGPlayerWindow::handle_fastmode(PG_RadioButton* button) {
  SetFastMode(button->GetPressed());
  return true;
}

bool PG_MPEGPlayerWindow::handle_reposition(PG_Slider* slider, long data) {
  SMPEGstatus v = cstat();
  double setp;
  setp = ((double)(data - slider->GetMinRange()) /
	  (double)(slider->GetMaxRange() - slider->GetMinRange()));
  if (v == SMPEG_PLAYING) Stop();
  Seek(setp * totalsize());
  if (v == SMPEG_PLAYING) Play();
  return true;
};

void PG_MPEGPlayerWindow::frameupdate() {
  PG_MPEGPlayer::frameupdate();
  if (pos != NULL) {
    double v = currenttime() / totaltime();
    pos->SetPosition(v * PLAYERSCALE);
  };
};

std::map<SDL_Surface*, PG_MPEGPlayer *> PG_MPEGPlayer::hash;

PG_MPEGPlayer::PG_MPEGPlayer(const char *file, PG_Widget* parent, const PG_Rect& rect) 
  : PG_Widget(parent, rect, true) {
  SetDirtyUpdate(true);
  f = SMPEG_new(file, &info, true);
  SMPEG_setdisplay(f, my_srfObject, NULL, cb);
  hash[my_srfObject] = this;
  currentscreen = my_srfObject;
  bFastMode = false;
}

void PG_MPEGPlayer::Play(void)   { SMPEG_play(f);   };
void PG_MPEGPlayer::Stop(void)   { SMPEG_stop(f);   };
void PG_MPEGPlayer::Rewind(void) { SMPEG_rewind(f); };
void PG_MPEGPlayer::Pause(void)  { SMPEG_pause(f);  };
void PG_MPEGPlayer::Seek(int bytes) { SMPEG_seek(f, bytes); };
void PG_MPEGPlayer::Skip(float seconds) { SMPEG_skip(f, seconds); };

PG_MPEGPlayer::~PG_MPEGPlayer() {
  Stop();
  SMPEG_delete(f);
  hash.erase(my_srfObject);
}

void PG_MPEGPlayer::cb(SDL_Surface* dst, int x, int y, unsigned int w, unsigned int h) {
  PG_Rect rsrc;
  PG_Rect rdst;
  
  PG_Application::LockScreen();
  PG_MPEGPlayer *player = hash[dst];
  if(player->bFastMode) {
    rsrc.SetRect(x, y, w, h);
    rdst = *(player->GetClipRect());
    SDL_BlitSurface(dst, &rsrc, player->my_srfScreen, &rdst);
    SDL_UpdateRects(player->my_srfScreen, 1, &rdst);
  }
  else {
    player->Update();
  }
  player->frameupdate();
  
  PG_Application::UnlockScreen();		
}

void PG_MPEGPlayer::frameupdate() { SMPEG_getinfo(f, &info); };
void PG_MPEGPlayer::SetFastMode(bool fast) { bFastMode = fast; }

void PG_MPEGPlayer::eventSizeWidget(Uint16 w, Uint16 h)
{
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  PG_Widget::eventSizeWidget(w,h);
#if 0
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  SMPEGstatus v = cstat();
  if (v == SMPEG_PLAYING) Stop();
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  SMPEG_setdisplay(f, my_srfObject, NULL, cb);
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  hash[my_srfObject] = this;
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  SMPEG_scaleXY(f,w,h);
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  if (v == SMPEG_PLAYING) Play();
#endif
};

bool PG_MPEGPlayer::SizeWidget(Uint16 w, Uint16 h)
{
  fprintf(stderr, "%s:%s:%i %i,%i\n", __FILE__, __FUNCTION__, __LINE__, w, h);
  bool rv = PG_Widget::SizeWidget(w,h);
  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
  // if (rv)
    {
      SMPEGstatus v = cstat();
      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
      if (v == SMPEG_PLAYING) Pause();
      // screen = SDL_SetVideoMode(event.resize.w, event.resize.h, screen->format->BitsPerPixel, screen->flags);
      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
      if (my_srfObject != currentscreen)
	{
	  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	  hash.erase(currentscreen);
	  currentscreen = my_srfObject;
	  hash[my_srfObject] = this;
	  SMPEG_setdisplay(f, my_srfObject, NULL, cb);
	};
      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
      SMPEG_scaleXY(f, my_srfObject->w, my_srfObject->h);
      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
      if (v == SMPEG_PLAYING) SMPEG_pause(f);
      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
    };
  return rv;
};
