/* pgmpeg2.hh
 * ----------
    PGMPEG - MPEG1 player library for ParaGUI
    Copyright (C) 2002  Eric Ross, Alexander Pipelka, Teunis Peters

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Eric Ross <eross@disc.ucn.cl>
    Alexander Pipelka <pipelka@teleweb.at>
    Teunis Peters <teunis@greycloaklabs.ca>
 */

#ifndef PGMPEG2_HH
#define PGMPEG2_HH

#include "pgapplication.h"
#include "pgwidget.h"
#include "pgbutton.h"
#include "pgwindow.h"
#include "pgslider.h"
#include "pgcheckbutton.h"

#include "smpeg/MPEG.h"
#include "smpeg/MPEGfilter.h"
#include "smpeg/smpeg.h"

#include <map>

/**
    @author Eric Ross, Alexander Pipelka, Teunis Peters
    
    @short A widget to display MPEG1 streams
    
    This widget displays MPEG1 video streams. It behaves exactly
    like other widgets except that you see the videostream inside ;-)
    The widget can be embedded into others and overlapping is fully
    supported.
*/

class PG_MPEGPlayer : public PG_Widget {
public:
	/**
	Constructor of the MPEG player
	@param file	path to videostream
	@param parent	pointer to parent widget
	@param rect	position of the widget
	
	Every PG_MPEGPlayer object constructs an associated MPEG (smpeg)
	object internally.
	*/
        PG_MPEGPlayer(const char *file, PG_Widget* parent, const PG_Rect& rect);

	/**
	Destruct the widget
	
	Cleans up the widget and deletes the internally created MPEG object
	*/
        ~PG_MPEGPlayer();

	/**
	Accelerate the video display
	@param fast	if true the display will be faster
	
	This function accelerates the video display by bypassing all
	widget clipping and overlapping functions. If you enable this
	option the video will always be displayed on top of all other
	widgets.
	*/
        void SetFastMode(bool fast);

        static void cb(SDL_Surface* dst, int x, int y, unsigned int w, unsigned int h);

        /**
	The MPEG player data
	*/
        SMPEG* f;
        SMPEG_Info info;

	/**
	Play the video
	*/
        void Play(void);

	/**
	Stop the video
	*/
        void Stop(void);

	/**
	Rewind the video back to it's beginning
	*/
        void Rewind(void);

        /**
	Pause/Resume playback of the video
	*/
        void Pause(void);

        /**
	Seek 'bytes' bytes in the MPEG stream
	@param bytes	distance (in bytes) where to reposition stream
	*/
        void Seek(int bytes);

        /**
	Skip 'seconds' seconds in the MPEG stream
	@param seconds	time (in seconds) where to reposition stream
	*/
        void Skip(float seconds);
  
        /**
	returns total size of video (in bytes)
	*/
        Uint32 totalsize() { return info.total_size; };

        /**
	returns total time of video (in seconds)
	*/
        double totaltime() { return info.total_time; };

        /**
	current video status

	return the current status (SMPEG_ERROR | SMPEG_STOPPED | SMPEG_PLAYING)
	of the video
	*/
        SMPEGstatus cstat() { return SMPEG_status(f); };

        /**
	returns the current offset (in bytes) of the videostream position
	*/
        Uint32 currentoffset() { return info.current_offset; };

        /**
	returns the current time (in seconds) of the videostream position
	*/
        double currenttime() { return info.current_time; };

        /**
	catch this event as it can break the mpeg player
	*/
        bool SizeWidget(Uint16 w, Uint16 h);
	void eventSizeWidget(Uint16 w, Uint16 h);

protected:
        /**
	Video frame update function

	an update that happens every frame.
	this one just updates the 'info' record.
	*/
        virtual void frameupdate();

private:
        static std::map<SDL_Surface*, PG_MPEGPlayer *> hash;
        bool bFastMode;
        SDL_Surface *currentscreen;
};

class PG_MPEGPlayerWindow : public PG_MPEGPlayer {
protected:
        PG_Slider* pos;
public:
        PG_MPEGPlayerWindow(const char *file, PG_Widget *parent, const PG_Rect & r)
	  : PG_MPEGPlayer(file, parent, r) {
	  pos=NULL;
	};
  
        ~PG_MPEGPlayerWindow() { };

        /* this slider will be updated with current player position */
        void handle_position(PG_Slider* slider);

        /* access sigc++ 'slots' *g* */
        bool handle_start(PG_Button* button);
        bool handle_stop(PG_Button* button);
        bool handle_rewind(PG_Button* button);
        bool handle_pause(PG_Button* button);
        bool handle_transparency(PG_Slider* slider, long data);
        bool handle_fastmode(PG_RadioButton* button);
        bool handle_reposition(PG_Slider* slider, long data);

protected:
        void frameupdate();
};

class PG_MoveableMPEGPLayerWindow : public PG_MPEGPlayerWindow {
        bool bDragMode;
public:
        PG_MoveableMPEGPLayerWindow(const char *file, PG_Widget *parent, const PG_Rect & r)
	  : PG_MPEGPlayerWindow(file, parent, r) {
	  bDragMode = false;
	};

        ~PG_MoveableMPEGPLayerWindow() { };

protected:
        bool eventMouseButtonDown(const SDL_MouseButtonEvent *button) {
	  bDragMode = true;
	  StartWidgetDrag();
	  SetCapture();
	  return true;
	}
  
        bool eventMouseButtonUp(const SDL_MouseButtonEvent *button) {
	  if(bDragMode)
	    {
	      EndWidgetDrag(button->x, button->y);
	      ReleaseCapture();
	    }
	  bDragMode = false;
	  return true;
	}
  
        bool eventMouseMotion (const SDL_MouseMotionEvent *motion) {
	  if(!bDragMode) { return true; }
	  WidgetDrag(motion->x, motion->y);
	  return true;
	}
};

#endif
